# Generated by Django 4.2.8 on 2023-12-22 21:51

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Selinux',
            fields=[
                ('hostname', models.CharField(max_length=128, primary_key=True, serialize=False)),
                ('status', models.CharField(max_length=50)),
                ('mount', models.CharField(max_length=50)),
                ('rootdir', models.CharField(max_length=50)),
                ('policyname', models.CharField(max_length=50)),
                ('current_mode', models.CharField(max_length=50)),
                ('configured_mode', models.CharField(max_length=50)),
                ('mslstatus', models.CharField(max_length=50)),
                ('memprotect', models.CharField(max_length=50)),
                ('maxkernel', models.CharField(max_length=50)),
                ('total', models.CharField(max_length=50)),
                ('success', models.CharField(max_length=50)),
                ('failed', models.CharField(max_length=50)),
                ('sealerts', models.CharField(max_length=50)),
            ],
            options={
                'verbose_name': 'Selinux',
                'verbose_name_plural': 'Selinux',
                'db_table': 'selinux',
                'ordering': ['hostname'],
            },
        ),
        migrations.CreateModel(
            name='SElinuxEvent',
            fields=[
                ('digest', models.CharField(max_length=256, primary_key=True, serialize=False)),
                ('hostname', models.CharField(max_length=128)),
                ('event', models.CharField(max_length=1024)),
                ('date', models.DateField()),
                ('time', models.TimeField()),
                ('serial_num', models.IntegerField()),
                ('event_kind', models.CharField(blank=True, max_length=256, null=True)),
                ('session', models.CharField(blank=True, max_length=256, null=True)),
                ('subj_prime', models.CharField(blank=True, max_length=256, null=True)),
                ('subj_sec', models.CharField(blank=True, max_length=256, null=True)),
                ('subj_kind', models.CharField(blank=True, max_length=256, null=True)),
                ('action', models.CharField(blank=True, max_length=256, null=True)),
                ('result', models.CharField(blank=True, max_length=256, null=True)),
                ('obj_prime', models.CharField(blank=True, max_length=256, null=True)),
                ('obj_sec', models.CharField(blank=True, max_length=256, null=True)),
                ('obj_kind', models.CharField(blank=True, max_length=256, null=True)),
                ('how', models.CharField(blank=True, max_length=256, null=True)),
            ],
            options={
                'verbose_name': 'SElinuxEvent',
                'verbose_name_plural': 'SElinuxEvent',
                'db_table': 'selinux_event',
                'ordering': ['date', 'time', 'hostname'],
            },
        ),
    ]
